@@ -220,7 +220,22 @@ def cnet_graph_from_tensor(cnet, delete_reltypes=None,
 
     return cnet_graph
 
-''' These are utility functions used by CrossBridge '''
+def graph_from_triples(triples, omit_relations=None,
+                       min_weight=0):
+    '''
+    Make a SemanticNetwork out of a sequence of triples.
+
+    You can get such a sequence from
+    csc.conceptnet4.analogyspace.conceptnet_triples.
+    '''
+    graph = SemanticNetwork()
+    omit_relations = set(omit_relations or [])
+    for (c1, rel, c2), val in triples:
+        if val < min_weight or rel in omit_relations: continue
+        graph.add_edge(c1, c2, rel)
+    return graph
+
+''' these are utility functions used by CrossBridge '''
 
 def k_subset_iter(k, in_set):
     '''
@@ -274,26 +289,28 @@ def k_edge_subgraphs(graph, min_subgraph_edges, num_subgraph_vertices, logging_i
     graph - the graph from which subgraphs should be extracted
     num_subgraph_vertices - The number of vertices in the subgraphs
     min_subgraph_edges - The minimum number of edges in each returned subgraph. min_subgraph_edges must be at least num_subgraph_vertices-1 (so the graph is connected)
+
+    Returns a dictionary mapping (num vertices, num edges) tuples to
+    sets of vertices, expressed as frozensets.
     '''
     assert(min_subgraph_edges >= num_subgraph_vertices-1)
 
     found_subgraphs = defaultdict(set)
 
-    ''' Base case: the set of 2 vertex subgraphs connected by 1 edge '''
-    found_subgraphs[(2, 1)] = []
-    for v1, v2, type in graph.edges():
-        found_subgraphs[(2, 1)].append(frozenset([v1, v2]))
+    # Base case: the set of 2-vertex subgraphs connected by 1 edge
+    found_subgraphs[(2, 1)] = set(frozenset([v1, v2])
+                                  for v1, v2, type in graph.edges())
 
     logging_counter = 0
     for num_nodes in xrange(3,num_subgraph_vertices + 1):
         logging.debug("Finding graphs with %d vertices...", num_nodes)
         for i in xrange(num_nodes - 2, min(min_subgraph_edges, (num_nodes - 2)*(num_nodes - 1)/2 + 1)):
-            ''' Fill in found_subgraphs[num_nodes, i] '''
+            # Fill in found_subgraphs[num_nodes, i]
             logging.debug("Checking subgraphs with %d vertices and %d edges (total: %d)...",
                           num_nodes - 1, i, len(found_subgraphs[num_nodes - 1, i]))
             for soln_subgraph in found_subgraphs[num_nodes - 1, i]:
-                ''' Compute edge counts for vertices adjacent to soln_subgraph '''
-                vertex_counts = defaultdict(lambda: 0)
+                #Compute edge counts for vertices adjacent to soln_subgraph
+                vertex_counts = defaultdict(int)
                 for soln_vertex in soln_subgraph:
                     for v in graph.get_neighbors(soln_vertex):
                         if v not in soln_subgraph:
