@@ -1,5 +1,5 @@
 from csc.divisi.ordered_set import OrderedSet, IdentitySet, indexable_set
-from csc.divisi.tensor import View, TheEmptySlice
+from csc.divisi.tensor import View, TheEmptySlice, DictTensor, DenseTensor
 import copy
 import heapq
 from operator import itemgetter
@@ -31,9 +31,9 @@ def make_sparse_labeled_tensor(ndim, labels=None,
      'tfidf.T': use tf-idf, transposed (matrix is documents by terms)
      a class: adds that class as a layer.
     '''
-    from csc.divisi.tensor import DictTensor
     if labels is None: labels = [OrderedSet() for _ in xrange(ndim)]
     tensor = LabeledView(DictTensor(ndim), labels)
+    tensor.tensor._shape[:] = [len(label_list) for label_list in labels]
     if initial is not None:
         tensor.update(initial)
     for k, v in accumulate or []:
@@ -46,7 +46,6 @@ def make_sparse_labeled_tensor(ndim, labels=None,
 
 
 def make_dense_labeled_tensor(data, labels):
-    from csc.divisi.tensor import DenseTensor
     return LabeledView(DenseTensor(data), labels)
 
 
@@ -136,12 +135,7 @@ class LabeledView(LabelingMixin, View):
         self._labels = [indexable_set(l, shape[mode])
                         for mode, l in enumerate(label_lists)]
 
-        #for i in xrange(self.ndim):
-        #    pass
-        # FIXME: actually check labels. We run into trouble when the underlying
-        # thing is a LabeledView. Perhaps we subclass LabeledView for that?
-#            if len(self._labels[i]) != shape[i]:
-#                raise IndexError("Bad labels for dimension %d" % i)
+        self.tensor.ensure_index_is_valid(tuple(len(label_list)-1 for label_list in self._labels))
 
 
     def __repr__(self):
@@ -289,10 +283,16 @@ class LabeledView(LabelingMixin, View):
         return self.layer_on(-self.tensor)
 
     def to_dense(self):
+        '''Change the underlying representation to a dense tensor.'''
         return self.layer_on(self.tensor.to_dense())
-
+    
     def to_sparse(self):
+        '''Change the underlying representation to a sparse tensor.'''
         return self.layer_on(self.tensor.to_sparse())
+    
+    def bake(self):
+        '''Simplify the representation to a LabeledView of a DictTensor.'''
+        return make_sparse_labeled_tensor(ndim=self.ndim, initial=self.iteritems())
 
     def dot(self, other):
         """
@@ -361,20 +361,12 @@ class LabeledView(LabelingMixin, View):
     def layer_on(self, tensor):
         return LabeledView(tensor, self._labels)
 
-    def zero_mean_normalized(self, mode=0, prefix='-'):
-        # This is a special case, since we must duplicate labels
-        new_tensor = self.tensor.zero_mean_normalized(mode=mode)
-
-        new_labels = copy.deepcopy(self._labels)
-        # Duplicate labels, preserving the property that
-        # index(label)[mode] = index(prefix  label)[mode] - self.tensor.shape[mode]
-        for i in xrange(self.tensor.shape[mode]):
-            l = self.label(mode, i)
-            new_labels[mode].add(prefix + l)
-
-        return LabeledView(new_tensor, new_labels)
-
-
+    def unnormalized(self):
+        '''
+        Remove the topmost layer of normalization from this tensor stack.
+        '''
+        return self.layer_on(self.tensor.unnormalized())
+    
     def array_op(self, op, labels=None, *args, **kwargs):
         """
         Apply an operation to the tensor inside (which must be a
@@ -397,7 +389,7 @@ class LabeledView(LabelingMixin, View):
         labels = self.label_list(0)
         return dict((labels[i], float(self.tensor[i])) for i in xrange(self.shape[0]))
 
-    def top_items(self, n=10, largest=True, key=None, abs_val=None):
+    def top_items(self, n=10, largest=True, key=None):
         '''
         For each of the top n items with greatest values, return a pair of
         the item and its value.
@@ -410,12 +402,6 @@ class LabeledView(LabelingMixin, View):
         If the tensor is 1D, gives indices instead of one-item tuples.
         Always expresses the output in floats directly.
         '''
-        if abs_val is not None:
-            import warnings
-            warnings.warn(DeprecationWarning('abs_val is deprecated. use key=abs.'))
-            assert key is None
-            key = abs
-
         if key is not None:
             extreme_key = lambda ent: key(ent[1])
         else:
@@ -429,17 +415,6 @@ class LabeledView(LabelingMixin, View):
         else:
             return [(k, float(v)) for k, v in items]
 
-    def bake(self):
-        '''
-        Simplify the representation.
-        '''
-        return make_sparse_labeled_tensor(ndim=self.ndim, initial=self.iteritems())
-
-    def to_dense(self):
-        '''Change the underlying representation to a dense tensor.'''
-        return LabeledView(self.tensor.to_dense(), self._labels)
-
-
     def concatenate(self, other):
         concat_dense = self.tensor.concatenate(other.tensor)
         newlabels = OrderedSet(list(self.label_list(0)) +
@@ -460,20 +435,57 @@ class LabeledView(LabelingMixin, View):
         return means, labeled_clusters
 
     def label_histograms(self):
+        '''
+        Counts the number and magnitude of items in each row / column.
+
+        Returns (counts, magnitudes).
+        '''
         counts, magnitudes = self.tensor.label_histograms()
         return ([LabeledView(count, [labels]) for count, labels
                  in izip(counts, self.label_sets())],
                 [LabeledView(mag, [labels]) for mag, labels
                  in izip(magnitudes, self.label_sets())])
 
+    def squish(self, cutoff=5):
+        import numpy as np
+        dims = range(self.ndim)
+        counts = [np.zeros((self.shape[mode],)) for mode in dims]
+        for labelindex in self:
+            indices = self.indices(labelindex)
+            for mode in dims:
+                counts[mode][indices[mode]] += 1
+
+        print "done counting."
+        newtensor = make_sparse_labeled_tensor(ndim=self.ndim)
+        for labelindex, value in self.iteritems():
+            keep = True
+            indices = self.indices(labelindex)
+            for mode in dims:
+                if counts[mode][indices[mode]] < cutoff:
+                    keep = False
+                    break
+            if keep:
+                newtensor[labelindex] = value
+        return newtensor
+    
+    def means(self):
+        '''
+        Computes the labeled by-slice means for each dimension.
+        '''
+        means = self.tensor.means()
+        return [make_dense_labeled_tensor(means, [labels])
+                for means, labels in zip(means, self.label_lists())]
+
 
 from csc.divisi.svd import SVD2DResults
 class LabeledSVD2DResults(SVD2DResults):
     @classmethod
     def layer_on(cls, results, src):
+        if hasattr(src, 'label_lists'):
+            src = src.label_lists()
         return cls(
-            LabeledView(results.u, [src._labels[0], None]),
-            LabeledView(results.v, [src._labels[1], None]),
+            LabeledView(results.u, [src[0], None]),
+            LabeledView(results.v, [src[1], None]),
             results.svals)
 
     def save_pytables(self, filename, title='SVD results', filters=None, **kw):
@@ -509,7 +521,6 @@ class LabeledSVD2DResults(SVD2DResults):
     @classmethod
     def load_pytables(cls, filename, copy=False):
         import numpy as np
-        from csc.divisi.tensor import DenseTensor
         from csc.divisi.pyt_utils import get_pyt_handle
         fileh = get_pyt_handle(filename)
 
@@ -522,9 +533,54 @@ class LabeledSVD2DResults(SVD2DResults):
             return wrapped_tensor(name).labeled(list(getattr(root, label_name)))
         u = wrapped_labeled_tensor('u', 'u_labels')
         v = wrapped_labeled_tensor('v', 'v_labels')
-        svals = root.svals.read()
+        svals = wrapped_tensor('svals')
         svd = cls(u, v, svals)
         svd._weighted_u = wrapped_tensor('weighted_u').labeled(u.label_lists())
         svd._weighted_v = wrapped_tensor('weighted_v').labeled(v.label_lists())
-        svd._core = DenseTensor(root.core)
+        svd._core = wrapped_tensor('core')
         return svd
+
+    # Pickle also works for loading and saving.
+    def __getstate__(self):
+        return dict(
+            u=self.u,
+            v=self.v,
+            svals=self.svals)
+    def __setstate__(self, state):
+        self.u = state['u']
+        self.v = state['v']
+        self.svals = state['svals']
+        self.clear_cache()
+
+    def get_weighted_u(self):
+        return self.u.layer_on(DenseTensor(self.u.tensor.unwrap() * self.svals.unwrap()))
+    def get_weighted_v(self):
+        return self.v.layer_on(DenseTensor(self.v.tensor.unwrap() * self.svals.unwrap()))
+
+
+    def slice(self, k=None, rows=None, cols=None):
+        '''
+        Slice out a part of the SVD results, fewer svals, rows, and/or columns.
+
+        k : int - how many svals to keep. None -> keep all.
+        rows : [obj] - a list of rows to keep. None -> keep all.
+        cols : [obj] - a list of cols to keep. None -> keep all.
+        '''
+        row_labels = self.u.label_list(0)
+        col_labels = self.v.label_list(0)
+        if rows is not None:
+            rows = map(row_labels.index, rows)
+            row_labels = indexable_set(rows, len(rows))
+        if cols is not None:
+            cols = map(col_labels.index, rows)
+            col_labels = indexable_set(cols, len(cols))
+        return LabeledSVD2DResults.layer_on(super(LabeledSVD2DResults, self).slice(k, rows, cols), [row_labels, col_labels])
+
+    def unlabeled(self):
+        return SVD2DResults(self.u.unwrap(), self.v.unwrap(), self.svals)
+    def label_lists(self):
+        return self.u.label_list(0), self.v.label_list(0)
+        
+    def denormalized(self, norms):
+        return LabeledSVD2DResults.layer_on(self.unlabeled().denormalized(norms), self.label_lists())
+
