@@ -1,4 +1,5 @@
-from csc.divisi.tensor import View, InvalidLayeringException
+from csc.divisi.tensor import View
+from csc.divisi.exceptions import InvalidLayeringException
 
 EPSILON = 0.000000001   # for floating point comparisons
 
@@ -15,7 +16,10 @@ class BaseNormalizedView(View):
     input_is_numeric = True
     output_is_numeric = True
 
-    def __init__(self, tensor, mode=0):
+    # Default to normalizing on mode 0.
+    default_modes = 0
+
+    def __init__(self, tensor, mode=None):
         '''
         * tensor: tensor to view.
         * mode: which mode to normalize over (0 = rows, 1 = columns, etc.)
@@ -23,6 +27,10 @@ class BaseNormalizedView(View):
         if len(tensor) == 0:
             raise ValueError('Tensor ' + repr(tensor) + ' is empty.')
 
+        # If this isn't a MeanSubtractedView, the underlying tensor can't be a normalized view.
+        if not isinstance(self, MeanSubtractedView) and tensor.stack_contains(BaseNormalizedView):
+            raise TypeError('Already normalized; .unnormalized() first.')
+
         # The underlying tensor must be numeric.
         for part in tensor.example_key():
             if not isinstance(part, (int, long)):
@@ -43,12 +51,16 @@ class BaseNormalizedView(View):
         raise TypeError('Normalized views are read-only.')
 
     @property
-    def normalize_mode(self):
+    def normalize_modes(self):
         """
-        Which mode of the tensor is normalized (0 for rows, 1 for columns,
+        Which modes of the tensor is normalized (0 for rows, 1 for columns,
         etc.)
         """
-        return self._normalize_mode
+        modes = self._normalize_mode
+        if modes is None: modes = self.default_modes
+        if not isinstance(modes, (list, tuple)):
+            modes = [modes]
+        return modes
 
     @property
     def norms(self):
@@ -56,11 +68,18 @@ class BaseNormalizedView(View):
         A list of the normalization factors that were computed for each
         row/column/whatever.
         """
+        # FIXME: this method should be in a subclass only.
         return self._norms
 
     def item_changed(self, indices, oldvalue, newvalue):
+        # FIXME: nobody ever even complained that this doesn't
+        # work. Maybe change notification just isn't worth it, and
+        # normalized views are just immutable.
         self.update_norm(indices, oldvalue, newvalue)
 
+    def update_norm(self, indices, prev, current):
+        raise NotImplementedError
+
     def refresh_norms():
         """
         Compute all of the normalization factors from scratch.
@@ -69,6 +88,9 @@ class BaseNormalizedView(View):
         caller = inspect.getouterframes(inspect.currentframe())[1][3]
         raise NotImplementedError(caller + ' must be implemented in subclass')
 
+    def unnormalized(self):
+        return self.tensor
+
 
 class NormalizedView(BaseNormalizedView):
     '''A normalized view of a tensor.
@@ -104,16 +126,9 @@ class NormalizedView(BaseNormalizedView):
     You can also supply a list of modes to normalize over.
     '''
 
-    @property
-    def modes(self):
-        modes = self._normalize_mode
-        if not isinstance(modes, (list, tuple)):
-            modes = [modes]
-        return modes
-
     def refresh_norms(self):
         tensor = self.tensor
-        modes = self.modes
+        modes = self.normalize_modes
         norms = [zeros((tensor.shape[mode],)) for mode in modes]
         for key, value in self.tensor.iteritems():
             for mode in modes:
@@ -124,7 +139,7 @@ class NormalizedView(BaseNormalizedView):
         self._norms = norms
 
     def update_norm(self, indices, prev, current):
-        modes = self.modes
+        modes = self.normalize_modes
         # Handle growing the norms array.
         for i_mode, mode in enumerate(modes):
             index = indices[mode]
@@ -133,9 +148,10 @@ class NormalizedView(BaseNormalizedView):
             self._norms[i_mode][index] += (current*current - prev*prev)
 
     def __getitem__(self, indices):
+        # FIXME: this doesn't work well for slices.
         if not isinstance(indices, (list, tuple)):
             indices = (indices,)
-        modes = self.modes
+        modes = self.normalize_modes
         norm = self.base ** 2 + sum(self._norms[i_mode][indices[mode]]
                                     for i_mode, mode in enumerate(modes))
         data = self.tensor[indices]
@@ -143,8 +159,7 @@ class NormalizedView(BaseNormalizedView):
         return data / sqrt(norm)
 
     def iteritems(self):
-        modes = self._normalize_mode
-        if not isinstance(modes, (list, tuple)): modes = [modes]
+        modes = self.normalize_modes
         base_squared = self.base ** 2
         for indices, value in self.tensor.iteritems():
             norm = base_squared + sum(self._norms[i_mode][indices[mode]]
@@ -152,30 +167,59 @@ class NormalizedView(BaseNormalizedView):
             yield indices, value / sqrt(norm)
 
     ### Extraction and views
-    def normalized(self, mode=0):
-        return NormalizedView(self.tensor, mode)
-
-    def unnormalized(self):
-        return self.tensor
-
     def layer_on(self, tensor):
         # FIXME: This looks shady.
         return NormalizedView(self.tensor, self.modes)
 
     def _svd(self, **kw):
-        if self.normalize_mode == 0:
+        if list(self.normalize_modes) == [0]:
             return self.tensor._svd(row_factors=self.norms[0], **kw)
         return super(NormalizedView, self)._svd(**kw)
 
     def _isvd(self, **kw):
-        if self.normalize_mode == 0:
+        if list(self.normalize_modes) == [0]:
             return self.tensor._isvd(row_factors=self.norms[0], **kw)
         return super(NormalizedView, self)._isvd(**kw)
 
     # All other operations fall back to the tensor.
 
 
-class AvgNormalizedView(BaseNormalizedView):
+class MeanSubtractedView(BaseNormalizedView):
+    '''Subtracts out the mean of each row and/or column.
+
+    Uses offset parameters to the SVD to compute this efficiently.
+    '''
+    default_modes = 0
+    def refresh_norms(self):
+        self.means = self.tensor.means()
+
+    def __getitem__(self, indices):
+        # FIXME: slices.
+        data = self.tensor[indices]
+        for mode in self.normalize_modes:
+            # not in-place, in case of slicing.
+            data = data - self.means[mode][indices[mode]]
+        return data
+
+    # FIXME: iteritems is incorrect if the tensor is not dense, since
+    # unspecified items will probably be non-zero.
+
+    def _svd(self, **kw):
+        modes = self.normalize_modes
+        if not all(0 <= m <= 1 for m in modes):
+            raise NotImplementedError
+        if 0 in modes:
+            kw['offset_for_row'] = -self.means[0]
+        if 1 in modes:
+            kw['offset_for_col'] = -self.means[1]
+        print kw
+        return self.tensor._svd(**kw)
+
+    def _isvd(self, **kw):
+        raise NotImplementedError
+
+
+class OldAvgNormalizedView(BaseNormalizedView):
     '''A NormalizedView that makes the average to be zero, instead of using a Euclidean norm.'''
     def refresh_norms(self):
         tensor = self.tensor
@@ -190,9 +234,6 @@ class AvgNormalizedView(BaseNormalizedView):
                 norms[modes.index(mode)][key[mode]][1] += 1
         self._norms = norms
 
-    def update_norm(self, indices, prev, current):
-        raise NotImplementedError
-
     def __getitem__(self, indices):
         mode = self._normalize_mode
         if not isinstance(mode, list): modes = [mode]
@@ -222,7 +263,7 @@ class TfIdfView(BaseNormalizedView):
         '''
         self.transposed = transposed # before the super call because
                                      # super calls refresh_norms.
-        super(TfIdfView, self).__init__(tensor, 0) # We only support mode 0.
+        super(TfIdfView, self).__init__(tensor) # Modes don't really make sense.
 
     def __repr__(self):
         return 'TfIdfView(%r, transposed=%r)' % (self.tensor, self.transposed)
@@ -244,9 +285,6 @@ class TfIdfView(BaseNormalizedView):
             counts_for_document[document] += value
             num_docs_that_contain_term[term] += 1
 
-    def update_norm(self, indices, prev, current):
-        raise NotImplementedError
-
     def tf(self, term, document):
         if self.transposed:
             term_count = self.tensor[document, term]
@@ -264,86 +302,3 @@ class TfIdfView(BaseNormalizedView):
             term, document = indices
         return self.tf(term, document) * self.idf(term)
 
-
-class ZeroMeanView(View):
-    '''
-    This normalized view makes the mean of all values along the
-    specified mode equal to 0. The view accomplishes this by
-    "mirroring" the keys along the specified mode such that each key
-    along the mode has a "mirror" key. The "mirror" key has value -1 *
-    (value of original key).
-
-    For example, the tensor
-
-        0  1  2  4
-        0  0  3  0
-
-    mirrored along mode 0 is:
-
-        0  1  2  4
-        0  0  3  0
-        0 -1 -2 -4
-        0  0 -3  0
-    '''
-    # FIXME: It would be better to simply multiply the indices by 2.
-    # TODO: allow mirroring on multiple modes, e.g., all of them.
-    def __init__(self, tensor, mode):
-        self.duplicate_mode = mode
-        self.tensor = tensor
-        self.tensor_mode_len = tensor.shape[mode]
-
-    def _normalize_key(self, indices):
-        # 1d tensors don't take tuple indices...
-        if not (isinstance(indices, list) or isinstance(indices, tuple)):
-            indices = [indices]
-
-        # Indices larger than the dimension size get wrapped around
-        if indices[self.duplicate_mode] >= self.tensor_mode_len:
-            indices = list(indices)
-            indices[self.duplicate_mode] = indices[self.duplicate_mode] - self.tensor_mode_len
-
-        return tuple(indices)
-
-    def _duplicate_key(self, indices):
-        # 1d tensors don't take tuple indices...
-        if not (isinstance(indices, list) or isinstance(indices, tuple)):
-            indices = [indices]
-        indices = list(indices)
-        indices[self.duplicate_mode] = indices[self.duplicate_mode] + self.tensor_mode_len
-        return tuple(indices)
-
-    def _is_duplicate_key(self, indices):
-        # 1d tensors don't take tuple indices...
-        if not (isinstance(indices, list) or isinstance(indices, tuple)):
-            indices = [indices]
-
-        return indices[self.duplicate_mode] >= self.tensor_mode_len
-
-    def __getitem__(self, indices):
-        if self._is_duplicate_key(indices):
-            return -1 * self.tensor[self._normalize_key(indices)]
-        else:
-            return self.tensor[indices]
-
-    def __len__(self):
-        # The tensor duplicates all keys, so there are twice as many
-        # in this tensor as there are in the base tensor
-        return 2*len(self.tensor)
-
-    @property
-    def shape(self):
-        s = list(self.tensor.shape)
-        s[self.duplicate_mode] *= 2
-        return tuple(s)
-
-    def has_key(self, key):
-        return self._normalize_key(key) in self.tensor
-
-    def __iter__(self):
-        for k in self.tensor:
-            yield k
-            yield self._duplicate_key(k)
-
-    def __repr__(self):
-        return '<ZeroMeanView of %r, duplicating mode: %r>' % (
-            self.tensor, self.duplicate_mode)
