@@ -21,6 +21,8 @@ struct smat {
   long *pointr;  /* For each col (plus 1), index of first non-zero entry. */
   long *rowind;  /* For each nz entry, the row index. */
   double *value; /* For each nz entry, the value. */
+  double *offset_for_row;
+  double *offset_for_col;
 };
 
 /* Row-major dense matrix.  Rows are consecutive vectors. */
@@ -93,6 +95,9 @@ SMat svdConvertDtoS(DMat D);
 DMat svdTransposeD(DMat D);
 /* Transposes a sparse matrix (returning a new one) */
 SMat svdTransposeS(SMat S);
+/* Copy a vector */
+double *copyVector(double* vec, int n, const char* name);
+
 
 /* Performs the las2 SVD algorithm and returns the resulting Ut, S, and Vt. */
 extern SVDRec svdLAS2(SMat A, long dimensions, long iterations, double end[2], 
