@@ -1,6 +1,6 @@
 import unittest
 from csc.divisi.tensor import DictTensor, DenseTensor
-from csc.divisi.normalized_view import NormalizedView, TfIdfView, ZeroMeanView
+from csc.divisi.normalized_view import NormalizedView, TfIdfView
 from csc.divisi.labeled_view import LabeledView, make_sparse_labeled_tensor
 from csc.divisi.ordered_set import OrderedSet, IdentitySet
 from csc.divisi.unfolded_set import UnfoldedSet
@@ -269,66 +269,6 @@ class LabeledDictTensorTest(unittest.TestCase):
         t3 = t1 + t2
         # FIXME: FINISH THIS!
 
-normalize_testcase = [[1, None],
-                      [3, 4]]
-
-normalize_expected_result = [[1, None],
-                             [3/5., 4/5.]]
-
-class NormalizedViewTest(unittest.TestCase):
-    def setUp(self):
-        self.raw = DictTensor(2)
-        self.raw.update(nones_removed(nested_list_to_dict(normalize_testcase)))
-        self.tensor = NormalizedView(self.raw, 0)
-
-    def test_result(self):
-        assertTensorEqual(self.tensor, normalize_expected_result)
-
-    def test_contains(self):
-        self.assertTrue((0,0) in self.tensor)
-        self.assertTrue(self.tensor.has_key((0,0)))
-        self.assertFalse((0,1) in self.tensor)
-        self.assertFalse(self.tensor.has_key((0,1)))
-
-class ZeroMeanViewTest(unittest.TestCase):
-    def setUp(self):
-        self.tensor = DictTensor(ndim=2)
-        self.tensor[0, 3] = 2
-        self.tensor[3, 4] = 3
-        self.tensor = self.tensor.zero_mean_normalized(mode=0)
-
-        self.labeled_tensor = SparseLabeledTensor(ndim=2)
-        self.labeled_tensor['a', 'b'] = 2
-        self.labeled_tensor['b', 'c'] = 3
-        self.labeled_tensor = self.labeled_tensor.zero_mean_normalized(mode=0, prefix='+')
-
-    def test_keys(self):
-        self.assertEqual(len(self.tensor), 4)
-        self.assertEqual(self.tensor.shape, (8, 5))
-        self.assertEqual(self.tensor.keys(),
-                         [(0, 3), (4, 3),
-                          (3, 4), (7, 4)])
-
-    def test_getitem(self):
-        self.assertEqual(self.tensor[0, 3], 2)
-        self.assertEqual(self.tensor[4, 3], -2)
-        self.assertEqual(self.tensor[3, 4], 3)
-        self.assertEqual(self.tensor[7, 4], -3)
-
-    def test_labeled_keys(self):
-        self.assertEqual(len(self.labeled_tensor), 4)
-        self.assertEqual(self.labeled_tensor.shape, (4, 2))
-        self.assertEqual(self.labeled_tensor.keys(),
-                         [('a', 'b'), ('+a', 'b'),
-                          ('b', 'c'), ('+b', 'c')])
-
-    def test_labeled_getitem(self):
-        self.assertEqual(self.labeled_tensor['a', 'b'], 2)
-        self.assertEqual(self.labeled_tensor['+a', 'b'], -2)
-        self.assertEqual(self.labeled_tensor['b', 'c'], 3)
-        self.assertEqual(self.labeled_tensor['+b', 'c'], -3)
-
-
 
 class TfIdfTest(unittest.TestCase):
     def test(self):
@@ -559,109 +499,21 @@ class NormalizeTest(unittest.TestCase):
         self.normalized = NormalizedView(self.tensor, mode=0)
 
         self.randomtensor = DenseTensor(numpy.random.normal(size=(5, 8)))
-        self.randomnormal = NormalizedView(self.randomtensor, mode=0)
+        self.randomnormal_0 = NormalizedView(self.randomtensor, mode=0)
+        self.randomnormal_1 = NormalizedView(self.randomtensor, mode=1)
 
     def test_norms(self):
         self.assertAlmostEqual(self.normalized[0,0], -1.0)
         self.assertAlmostEqual(self.normalized[2,0], 0.99980006)
+
         for i in range(5):
-            row = [self.randomnormal[i,j] for j in range(8)]
+            row = [self.randomnormal_0[i,j] for j in range(8)]
             self.assertAlmostEqual(numpy.dot(row, row), 1.0)
 
-# Use a matrix with a known SVD in order to test the values
-# of the u / v / sigma matrices
-svd_2d_test_matrix = numpy.zeros((4, 5))
-svd_2d_test_matrix[0, 0] = 1
-svd_2d_test_matrix[0, 4] = 2
-svd_2d_test_matrix[1, 2] = 3
-svd_2d_test_matrix[3, 1] = 4
+        for i in range(8):
+            col = [self.randomnormal_1[i,j] for i in range(5)]
+            self.assertAlmostEqual(numpy.dot(col, col), 1.0)
 
-class SVD2DTest(unittest.TestCase):
-    def setUp(self):
-        self.tensor = DictTensor(2)
-        # Note: this command actually puts 20 values in tensor!
-        self.tensor.update(nested_list_to_dict(svd_2d_test_matrix))
-        self.svd = self.tensor.svd(k=3)
-        self.incremental = self.tensor.incremental_svd(k=3, niter=200)
-        self.u, self.svals, self.v = self.svd.u, self.svd.svals, self.svd.v
-
-    def test_incremental(self):
-        self.assertEqual(self.incremental.u.shape[0], self.tensor.shape[0])
-        self.assertEqual(len(self.incremental.svals), self.incremental.u.shape[1])
-        self.assertEqual(len(self.incremental.svals), self.incremental.v.shape[1])
-        self.assertEqual(self.incremental.v.shape[0], self.tensor.shape[1])
-
-        assertTensorEqual(self.incremental.u,
-                               [[0, 0, 1],
-                                [0, 1, 0],
-                                [0, 0, 0],
-                                [1, 0, 0]])
-
-        assertTensorEqual(self.incremental.v,
-                               [[0, 0, sqrt(.2)],
-                                [1, 0, 0],
-                                [0, 1, 0],
-                                [0, 0, 0],
-                                [0, 0, sqrt(.8)]])
-
-        assertTensorEqual(self.incremental.svals,
-                               [4, 3, sqrt(5)])
-
-    def test_decomposition(self):
-        self.assertEqual(self.u.shape[0], self.tensor.shape[0])
-        self.assertEqual(len(self.svals), self.u.shape[1])
-        self.assertEqual(len(self.svals), self.v.shape[1])
-        self.assertEqual(self.v.shape[0], self.tensor.shape[1])
-
-        assertTensorEqual(self.u,
-                               [[0, 0, 1],
-                                [0, -1, 0],
-                                [0, 0, 0],
-                                [-1, 0, 0]], abs=True)
-
-        assertTensorEqual(self.v,
-                               [[0, 0, sqrt(.2)],
-                                [-1, 0, 0],
-                                [0, -1, 0],
-                                [0, 0, 0],
-                                [0, 0, sqrt(.8)]], abs=True)
-
-        assertTensorEqual(self.svals,
-                               [4, 3, sqrt(5)])
-
-    def test_reconstructed(self):
-        assertTensorEqual(self.svd.reconstructed,
-                               [[1, 0, 0, 0, 2],
-                                [0, 0, 3, 0, 0],
-                                [0, 0, 0, 0, 0],
-                                [0, 4, 0, 0, 0]])
-        assertTensorEqual(self.svd.reconstructed[1,:],
-                                [0, 0, 3, 0, 0])
-        assertTensorEqual(self.svd.reconstructed[:,2],
-                               [0, 3, 0, 0])
-
-    def test_orthonormality(self):
-        identity = [[1, 0, 0],
-                     [0, 1, 0],
-                     [0, 0, 1]]
-        assertTensorEqual(self.u.T * self.u,
-                               identity)
-
-        assertTensorEqual(self.v.T * self.v,
-                               identity)
-
-    def test_variance(self):
-        # Assert that the SVD explained some of the variance.
-        diff_k3 = self.tensor - self.svd.reconstructed
-        tensor_mag = self.tensor.magnitude()
-        diff_k3_mag = diff_k3.magnitude()
-        self.assert_(tensor_mag > diff_k3_mag)
-
-        # Check that a smaller SVD explains less of the variance, but still some.
-        svd_k1 = self.tensor.svd(k=1)
-        diff_k1 = self.tensor - svd_k1.reconstructed
-        diff_k1_mag = diff_k1.magnitude()
-        self.assert_(tensor_mag > diff_k1_mag > diff_k3_mag)
 
 class NormalizedSVD2DTest(unittest.TestCase):
     def setUp(self):
